import type { BeeRequestOptions, ChequebookAddressResponse, ChequebookBalanceResponse, LastCashoutActionResponse, LastChequesForPeerResponse, LastChequesResponse, NumberString, TransactionOptions } from '../../types';
import { PeerAddress, TransactionId } from '../../utils/typed-bytes';
/**
 * Get the address of the chequebook contract used
 *
 * @param requestOptions Options for making requests
 */
export declare function getChequebookAddress(requestOptions: BeeRequestOptions): Promise<ChequebookAddressResponse>;
/**
 * Get the balance of the chequebook
 *
 * @param requestOptions Options for making requests
 */
export declare function getChequebookBalance(requestOptions: BeeRequestOptions): Promise<ChequebookBalanceResponse>;
/**
 * Get last cashout action for the peer
 *
 * @param requestOptions Options for making requests
 * @param peer  Swarm address of peer
 */
export declare function getLastCashoutAction(requestOptions: BeeRequestOptions, peer: PeerAddress): Promise<LastCashoutActionResponse>;
/**
 * Cashout the last cheque for the peer
 *
 * @param requestOptions Options for making requests
 * @param peer  Swarm address of peer
 * @param options
 */
export declare function cashoutLastCheque(requestOptions: BeeRequestOptions, peer: PeerAddress, options?: TransactionOptions): Promise<TransactionId>;
/**
 * Get last cheques for the peer
 *
 * @param requestOptions Options for making requests
 * @param peer  Swarm address of peer
 */
export declare function getLastChequesForPeer(requestOptions: BeeRequestOptions, peer: PeerAddress): Promise<LastChequesForPeerResponse>;
/**
 * Get last cheques for all peers
 *
 * @param requestOptions Options for making requests
 */
export declare function getLastCheques(requestOptions: BeeRequestOptions): Promise<LastChequesResponse>;
/**
 * Deposit tokens from overlay address into chequebook
 *
 * @param requestOptions Options for making requests
 * @param amount   Amount of tokens to deposit
 * @param gasPrice Gas Price in WEI for the transaction call
 * @return string  Hash of the transaction
 */
export declare function depositTokens(requestOptions: BeeRequestOptions, amount: NumberString | string | bigint, gasPrice?: NumberString | string | bigint): Promise<TransactionId>;
/**
 * Withdraw tokens from the chequebook to the overlay address
 *
 * @param requestOptions Options for making requests
 * @param amount   Amount of tokens to withdraw
 * @param gasPrice Gas Price in WEI for the transaction call
 * @return string  Hash of the transaction
 */
export declare function withdrawTokens(requestOptions: BeeRequestOptions, amount: NumberString | string | bigint, gasPrice?: NumberString | string | bigint): Promise<TransactionId>;
