import { BeeRequestOptions } from '../../index';
import type { DebugStatus, Health, NodeInfo, Readiness } from '../../types/debug';
import { BeeVersions } from '../../types/debug';
export declare const SUPPORTED_BEE_VERSION_EXACT = "2.4.0-390a402e";
export declare const SUPPORTED_BEE_VERSION: string;
export declare const SUPPORTED_API_VERSION = "7.2.0";
export declare function getDebugStatus(requestOptions: BeeRequestOptions): Promise<DebugStatus>;
/**
 * Get health of node
 *
 * @param requestOptions Options for making requests
 */
export declare function getHealth(requestOptions: BeeRequestOptions): Promise<Health>;
/**
 * Get readiness of node
 *
 * @param requestOptions Options for making requests
 */
export declare function getReadiness(requestOptions: BeeRequestOptions): Promise<Readiness>;
/**
 * Get information about Bee node
 *
 * @param requestOptions Options for making requests
 */
export declare function getNodeInfo(requestOptions: BeeRequestOptions): Promise<NodeInfo>;
/**
 * Connects to a node and checks if its version matches with the one that bee-js supports.
 *
 * This is the most strict version check and most probably you will
 * want to use the relaxed API-versions check `isSupportedApiVersion`.
 *
 * @param requestOptions Options for making requests
 */
export declare function isSupportedExactVersion(requestOptions: BeeRequestOptions): Promise<boolean>;
/**
 * Connects to a node and checks if its Main API versions matches with the one that bee-js supports.
 *
 * This should be the main way how to check compatibility for your app and Bee node.
 *
 * @param requestOptions Options for making requests
 */
export declare function isSupportedApiVersion(requestOptions: BeeRequestOptions): Promise<boolean>;
/**
 * Returns object with all versions specified by the connected Bee node (properties prefixed with `bee*`)
 * and versions that bee-js supports (properties prefixed with `supported*`).
 *
 * @param requestOptions Options for making requests
 */
export declare function getVersions(requestOptions: BeeRequestOptions): Promise<BeeVersions>;
