import { BeeRequestOptions, NumberString, TransactionInfo } from '../../types';
import { TransactionId } from '../../utils/typed-bytes';
/**
 * Get list of all pending transactions
 *
 * @param requestOptions Options for making requests
 */
export declare function getAllTransactions(requestOptions: BeeRequestOptions): Promise<TransactionInfo[]>;
/**
 * Get information for specific pending transactions
 *
 * @param requestOptions Options for making requests
 * @param transactionHash Hash of the transaction
 */
export declare function getTransaction(requestOptions: BeeRequestOptions, transactionHash: TransactionId): Promise<TransactionInfo>;
/**
 * Rebroadcast existing transaction
 *
 * @param requestOptions Options for making requests
 * @param transactionHash Hash of the transaction
 */
export declare function rebroadcastTransaction(requestOptions: BeeRequestOptions, transactionHash: TransactionId): Promise<TransactionId>;
/**
 * Cancel existing transaction
 *
 * @param requestOptions Options for making requests
 * @param transactionHash Hash of the transaction
 * @param gasPrice Optional gas price
 */
export declare function cancelTransaction(requestOptions: BeeRequestOptions, transactionHash: TransactionId, gasPrice?: NumberString | string | bigint): Promise<TransactionId>;
