/// <reference types="node" />
import * as stream from 'stream';
import { AllTagsOptions, BeeRequestOptions, CollectionUploadOptions, DownloadOptions, FileUploadOptions, GsocMessageHandler, NumberString, PostageBatchOptions, PssMessageHandler, RedundantUploadOptions, Tag, TransactionOptions, UploadOptions } from '../types';
export declare function isReadable(value: unknown): value is stream.Readable;
export declare function asNumberString(value: unknown, options?: {
    name?: string;
    min?: bigint;
    max?: bigint;
}): NumberString;
export declare function prepareBeeRequestOptions(value: unknown): BeeRequestOptions;
export declare function prepareDownloadOptions(value: unknown): DownloadOptions;
export declare function prepareUploadOptions(value: unknown, name?: string): UploadOptions;
export declare function prepareRedundantUploadOptions(value: unknown, name?: string): RedundantUploadOptions;
export declare function prepareFileUploadOptions(value: unknown): FileUploadOptions;
export declare function prepareCollectionUploadOptions(value: unknown): CollectionUploadOptions;
export declare function isTag(value: unknown): value is Tag;
export declare function preparePssMessageHandler(value: unknown): PssMessageHandler;
export declare function prepareGsocMessageHandler(value: unknown): GsocMessageHandler;
export declare function preparePostageBatchOptions(value: unknown): PostageBatchOptions;
export declare function prepareTransactionOptions(value: unknown, name?: string): TransactionOptions;
/**
 * Check whether the given parameter is valid data to upload
 * @param value
 * @throws TypeError if not valid
 */
export declare function assertData(value: unknown): asserts value is string | Uint8Array;
/**
 * Check whether the given parameter is a correct file representation to file upload.
 * @param value
 * @throws TypeError if not valid
 */
export declare function assertFileData(value: unknown): asserts value is string | Uint8Array | stream.Readable | File;
/**
 * Checks whether optional options for AllTags query are valid
 * @param options
 */
export declare function prepareAllTagsOptions(value: unknown): AllTagsOptions;
/**
 * Utility functions that return Tag UID
 * @param tagUid
 */
export declare function makeTagUid(tagUid: number | Tag | string | null | undefined): number;
