import { Bytes } from './bytes';
export declare class PrivateKey extends Bytes {
    static readonly LENGTH = 32;
    constructor(bytes: Uint8Array | string | Bytes);
    publicKey(): PublicKey;
    sign(data: Uint8Array | string): Signature;
}
export declare class PublicKey extends Bytes {
    static readonly LENGTH = 64;
    constructor(bytes: Uint8Array | string | Bytes);
    address(): EthAddress;
    toCompressedUint8Array(): Uint8Array;
    toCompressedHex(): string;
}
export declare class EthAddress extends Bytes {
    static readonly LENGTH = 20;
    constructor(bytes: Uint8Array | string | Bytes);
    toChecksum(): string;
}
export declare class Identifier extends Bytes {
    static readonly LENGTH = 32;
    constructor(bytes: Uint8Array | string | Bytes);
    static fromString(value: string): Identifier;
}
export declare class Reference extends Bytes {
    static readonly LENGTH = 32;
    constructor(bytes: Uint8Array | string | Bytes);
    toCid(type: 'feed' | 'manifest'): string;
    static isValid(value: string): boolean;
}
export declare class TransactionId extends Bytes {
    static readonly LENGTH = 32;
    constructor(bytes: Uint8Array | string | Bytes);
}
export declare class Span extends Bytes {
    static readonly LENGTH = 8;
    constructor(bytes: Uint8Array | string | Bytes);
    static fromBigInt(number: bigint): Span;
    toBigInt(): bigint;
    static fromSlice(bytes: Uint8Array, start: number): Span;
}
export declare class PeerAddress extends Bytes {
    static readonly LENGTH = 32;
    constructor(bytes: Uint8Array | string | Bytes);
}
export declare class BatchId extends Bytes {
    static readonly LENGTH = 32;
    constructor(bytes: Uint8Array | string | Bytes);
}
export declare class Signature extends Bytes {
    static readonly LENGTH = 65;
    constructor(bytes: Uint8Array | string | Bytes);
    static fromSlice(bytes: Uint8Array, start: number): Signature;
    recoverPublicKey(digest: Uint8Array | string): PublicKey;
    isValid(digest: Uint8Array | string, expectedAddress: EthAddress | Uint8Array | string): boolean;
}
export declare class Topic extends Bytes {
    static readonly LENGTH = 32;
    constructor(bytes: Uint8Array | string | Bytes);
    static fromString(value: string): Topic;
}
export declare class FeedIndex extends Bytes {
    static readonly LENGTH = 8;
    static readonly MINUS_ONE: FeedIndex;
    constructor(bytes: Uint8Array | string | Bytes);
    static fromBigInt(number: bigint): FeedIndex;
    toBigInt(): bigint;
    next(): FeedIndex;
}
